@@ -1,9 +1,8 @@
 package conf
 
 import (
-	"github.com/golang/protobuf/proto"
-
 	"github.com/xtls/xray-core/proxy/loopback"
+	"google.golang.org/protobuf/proto"
 )
 
 type LoopbackConfig struct {
